"""Azure Variable Object Class."""


class AzureVariable:
    """Creates Azure Variable Object."""

    def __init__(self, variable_name, variable_value):
        self.variable_name = variable_name
        self.variable_value = variable_value
